(function(){
    'use strict';

    const showLog = false;
    function log() {
        showLog && console.log.apply(console, arguments);
    }

    /**
     *
     * @param {string} selector
     * @param {Element} [root]
     * @returns {Array<Element>}
     */
    function selectAll(selector, root) {
        return Array.from((root || document).querySelectorAll(selector));
    }

    function selectOne(selector, root) {
        return (root || document).querySelector(selector);
    }

    function sendUpdateMessageToBackgroundScript(clearPrivateData = false) {
        chrome.runtime.sendMessage({
            stateUpdateMessage: true,
            clearPrivateData,
            aggressiveModeEnabled: isAggressiveModeEnabled()
        });
    }

    function clearPrivateData() {
        sendUpdateMessageToBackgroundScript(true);
    }

    function isAggressiveModeEnabled() {
        return !!selectOne('.mode-aggressive.on');
    }

    // Saves options to chrome.storage
    function saveUserConfig() {
        const aggressiveModeEnabled = isAggressiveModeEnabled();
        const opts = {
            "aggressiveModeEnabled": aggressiveModeEnabled,
        };
        if (opts.aggressiveModeEnabled) {
            opts.aggressiveModeAttemptedAtLeastOnce = true;
        }
        chrome.storage.sync.set(opts);
    }

    function getSavedOptions() {
        const aggressiveModeEnabledDefaultValue = false;
        return new Promise(resolve => {
            chrome.storage.sync.get({
                "aggressiveModeEnabled": aggressiveModeEnabledDefaultValue,
            }, resolve);
        });
    }

    function restoreUserConfig() {
        getSavedOptions().then(vals => {
            setCheckedRadio(selectOne(vals["aggressiveModeEnabled"] ? '.mode-aggressive' : '.mode-normal'));
            sendUpdateMessageToBackgroundScript();
        });
    }

    function showMessage(messageId) {
        selectAll('.info-message').forEach(el => el.hidden = true);
        selectOne('#' + messageId).hidden = false;
    }

    function setCheckedRadio(clickedElem) {
        selectAll('.custom-radio').forEach(el => {
            el.classList.remove('on');
            el.parentElement.classList.remove('active');
        });
        clickedElem.classList.add('on');
        clickedElem.parentElement.classList.add('active');
    }

    function init() {
        selectAll('.menu-item').forEach(el => {
            el.addEventListener('click', function(evt) {
                setCheckedRadio(this.querySelector('.custom-radio'));
                sendUpdateMessageToBackgroundScript();
                saveUserConfig();
            });
        });

        selectAll('[aria-describedBy]').forEach(el => {
            el.addEventListener('mouseenter', evt => {
                showMessage(el.getAttribute('aria-describedBy'));
            });
        });

        selectOne('#clearPrivateData').addEventListener('click', evt => {
            log('incog:' + chrome.extension.inIncognitoContext);
            if (chrome.extension.inIncognitoContext) {
                log('incog:' + chrome.extension.inIncognitoContext);
                clearPrivateData();
            }
        });
        // selectOne('#open-shortcuts-page').addEventListener('click', evt => {
        //     evt.preventDefault();
        //     chrome.tabs.create({ 'url': 'chrome://extensions/shortcuts'});
        // });

        // Disable the button, and change the tooltip.
        if (!chrome.extension.inIncognitoContext) {
            selectOne('[aria-describedby="msg-clear-private-data-ok"]').setAttribute('aria-describedby', 'msg-clear-private-data-disabled');
            selectOne('#clearPrivateData').disabled = true;
        }

        restoreUserConfig();
    }

    document.addEventListener('DOMContentLoaded', init);
})();